"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatDataValues = exports.transformKeys = exports.parseRequestData = exports.getDeviceID = void 0;
var config_1 = require("../config");
function getDeviceID(inputJson) {
    var _a;
    var data = JSON.parse(inputJson);
    var deviceID = data.key || ((_a = data.result) === null || _a === void 0 ? void 0 : _a.id) || '';
    return deviceID;
}
exports.getDeviceID = getDeviceID;
var keysMap = {
    cur_current: 'current',
    cur_voltage: 'voltage',
    cur_power: 'power',
    switch_1: 'power_switch',
    battery_percentage: 'battery',
    va_temperature: 'temperature',
    temp_current: 'temperature',
    va_humidity: 'humidity',
    humidity_value: 'humidity',
    bright_value_v2: 'white_brightness',
    bright_value: 'white_brightness',
    temp_value_v2: 'white_temperature',
    temp_value: 'white_temperature',
    colour_data_v2: 'color_data',
    scene_data_v2: 'scene_data',
    colour_data: 'color_data',
};
// function used to extract changed properties of a device - works for StatusReport, GET-Request and DevicesProperties messages from the Tuya Message Service
function parseRequestData(inputJson, messageType) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    var data = JSON.parse(inputJson);
    switch (messageType) {
        case config_1.TuyaMessageType.DEVICE_PROPERTY_MESSAGE:
            // Assuming status is an array of objects, each with a 'code' and a 'value'
            var properties = ((_a = data.result) === null || _a === void 0 ? void 0 : _a.status) || ((_c = (_b = data.payload) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.status) || ((_f = (_e = (_d = data.payload) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e.bizData) === null || _f === void 0 ? void 0 : _f.properties) || [];
            var simplifiedProperties = properties.reduce(function (acc, _a) {
                var code = _a.code, value = _a.value;
                acc[code] = value;
                return acc;
            }, {});
            return formatDataValues(transformKeys(simplifiedProperties, keysMap));
        case config_1.TuyaMessageType.DEVICE_ONLINE:
            return { online: true };
        case config_1.TuyaMessageType.DEVICE_OFFLINE:
            return { online: false };
        default:
            // Assuming status is an array of objects, each with a 'code' and a 'value'
            var defaultProperties = ((_g = data.result) === null || _g === void 0 ? void 0 : _g.status) || ((_j = (_h = data.payload) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.status) || ((_m = (_l = (_k = data.payload) === null || _k === void 0 ? void 0 : _k.data) === null || _l === void 0 ? void 0 : _l.bizData) === null || _m === void 0 ? void 0 : _m.properties) || [];
            var defaultsimplifiedProperties = defaultProperties.reduce(function (acc, _a) {
                var code = _a.code, value = _a.value;
                acc[code] = value;
                return acc;
            }, {});
            return formatDataValues(transformKeys(defaultsimplifiedProperties, keysMap));
    }
}
exports.parseRequestData = parseRequestData;
function transformKeys(properties, keyMap) {
    // Create a shallow copy of the object to avoid mutating the original object
    var transformedProperties = __assign({}, properties);
    // Iterate over the keyMap
    Object.entries(keyMap).forEach(function (_a) {
        var originalKey = _a[0], newKey = _a[1];
        // Check if the original key exists in the object
        if (originalKey in transformedProperties) {
            // Assign the value to the new key
            transformedProperties[newKey] = transformedProperties[originalKey];
            // Delete the original key
            delete transformedProperties[originalKey];
        }
    });
    return transformedProperties;
}
exports.transformKeys = transformKeys;
function formatDataValues(properties) {
    var formattedProperties = __assign({}, properties);
    Object.entries(formattedProperties).forEach(function (_a) {
        var code = _a[0], value = _a[1];
        switch (code) {
            case 'voltage':
                formattedProperties[code] = value / 10;
                break;
            case 'power_switch':
                formattedProperties[code] = value.toString() == 'true' ? 'power_on' : 'power_off';
                break;
            case 'temperature':
                formattedProperties[code] = value / 10;
                break;
            case 'doorcontact_state':
                formattedProperties[code] =
                    value.toString() == 'true' ? 'open' : value.toString() == 'false' ? 'closed' : 'N/A';
                break;
            case 'white_brightness':
                formattedProperties[code] = value / 10;
                break;
        }
    });
    return formattedProperties;
}
exports.formatDataValues = formatDataValues;
